package com.aghajari.axrlottiemoji;

import android.content.Context;
import android.graphics.Bitmap;

import android.graphics.Paint;
import android.text.Spannable;
import android.text.SpannableStringBuilder;
import android.text.TextUtils;
import android.text.style.DynamicDrawableSpan;
import android.text.style.ImageSpan;
import android.util.AttributeSet;

import com.aghajari.emojiview.utils.EmojiRange;
import com.aghajari.emojiview.view.AXEmojiTextView;
import com.aghajari.rlottie.AXrLottieDrawable;

import java.util.ArrayList;
import java.util.List;

public class AXrLottieTextView extends AXEmojiTextView {

    private final List<AXrLottieDrawable> drawables = new ArrayList<>();

    public AXrLottieTextView(Context context) {
        super(context);
    }

    public AXrLottieTextView(Context context, AttributeSet attrs) {
        super(context, attrs);
    }

    @Override
    public void setText(CharSequence rawText, BufferType type) {
        try {
            super.setText(replaceWithLottie(rawText), type);
        }catch (Exception ignore) {
            // LOADING THE PROVIDER
            post(() -> setText(replaceWithLottie(rawText), type));
        }
    }

    private CharSequence replaceWithLottie(CharSequence rawText) {
        recycleDrawables();

        if (TextUtils.isEmpty(rawText))
            return rawText;

        final SpannableStringBuilder text =
                new SpannableStringBuilder(rawText);

        AXrLottieEmojiSpan[] existingSpans = text.getSpans(0, text.length(), AXrLottieEmojiSpan.class);
        final List<Integer> existingSpanPositions = new ArrayList<>(existingSpans.length);
        final int size = existingSpans.length;
        //noinspection ForLoopReplaceableByForEach
        for (int i = 0; i < size; i++) {
            existingSpanPositions.add(text.getSpanStart(existingSpans[i]));
            attach(existingSpans[i]);
        }

        List<EmojiRange> findAllEmojis = AXrLottieEmojiManager.findAllLottieEmojis(text);

        for (int i = 0; i < findAllEmojis.size(); i++) {
            final EmojiRange location = findAllEmojis.get(i);

            if (!existingSpanPositions.contains(location.start)) {
                AXrLottieEmojiSpan span = new AXrLottieEmojiSpan(
                        getContext(),
                        (AXrLottieEmoji) location.emoji,
                        DynamicDrawableSpan.ALIGN_BOTTOM,
                        getPaint().getFontMetrics()
                );
                text.setSpan(span, location.start, location.end, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
                attach(span);
            }
        }

        return text;
    }

    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        if (getVisibility() == VISIBLE)
        startAll();
    }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        stopAll();
    }

    @Override
    public void setVisibility(int visibility) {
        int old = getVisibility();
        super.setVisibility(visibility);
        if (old != visibility) {
            if (visibility == VISIBLE)
                startAll();
            else
                stopAll();
        }
    }

    private void stopAll() {
        if (drawables == null)
            return;

        for (AXrLottieDrawable d : drawables)
            d.stop();
    }

    private void startAll() {
        if (drawables == null)
            return;

        for (AXrLottieDrawable d : drawables)
            d.start();
    }

    private void recycleDrawables() {
        if (drawables == null)
            return;

        for (AXrLottieDrawable d : drawables)
            d.setOnFrameRenderListener(null);
        drawables.clear();
    }

    private void attach(AXrLottieEmojiSpan span) {
        AXrLottieDrawable drawable = ((AXrLottieDrawable) span.getDrawable());
        drawable.setOnFrameRenderListener(new AXrLottieDrawable.OnFrameRenderListener() {
            @Override
            public void onUpdate(AXrLottieDrawable drawable, int frame, long timeDiff, boolean force) {
            }

            @Override
            public Bitmap renderFrame(AXrLottieDrawable drawable, Bitmap bitmap, int frame) {
                postInvalidate();
                return null;
            }
        });
        drawables.add(drawable);
    }

    static class AXrLottieEmojiSpan extends ImageSpan {
        private final Paint.FontMetrics fontMetrics;
        private final float density;
        private final int size;

        public AXrLottieEmojiSpan(Context context, AXrLottieEmoji emoji, int verticalAlignment, Paint.FontMetrics original) {
            super(emoji.getDrawable(context, calculateSize(context, original)), verticalAlignment);
            density = context.getResources().getDisplayMetrics().density;
            size = calculateSize(context, original);

            fontMetrics = original;
        }

        @Override
        public int getSize(Paint paint, CharSequence text, int start, int end, Paint.FontMetricsInt fm) {
            if (fm == null) {
                fm = new Paint.FontMetricsInt();
            }

            if (fontMetrics == null) {
                int sz = super.getSize(paint, text, start, end, fm);

                int offset = (int) (density * 8);
                int w = (int) (density * 10);
                fm.top = -w - offset;
                fm.bottom = w - offset;
                fm.ascent = -w - offset;
                fm.leading = 0;
                fm.descent = w - offset;

                return sz;
            } else {
                fm.ascent = (int) fontMetrics.ascent;
                fm.descent = (int) fontMetrics.descent;

                fm.top = (int) fontMetrics.top;
                fm.bottom = (int) fontMetrics.bottom;
                if (getDrawable() != null)
                    getDrawable().setBounds(0, 0, size, size);

                return size;
            }
        }
    }

    private static int calculateSize(Context context, Paint.FontMetrics fontMetrics) {
        int size = 0;
        if (fontMetrics != null)
            size = (int) (Math.abs(fontMetrics.descent) + Math.abs(fontMetrics.ascent));
        return size == 0 ? (int) (context.getResources().getDisplayMetrics().density * 20) : size;
    }
}
